from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Literal, cast






T = TypeVar("T", bound="PostByOrgByRepoCommitBodyFilesItemType1")



@_attrs_define
class PostByOrgByRepoCommitBodyFilesItemType1:
    """ 
        Attributes:
            path (str):
            action (Literal['delete']):
     """

    path: str
    action: Literal['delete']
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        path = self.path

        action = self.action


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "path": path,
            "action": action,
        })

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        path = d.pop("path")

        action = cast(Literal['delete'] , d.pop("action"))
        if action != 'delete':
            raise ValueError(f"action must match const 'delete', got '{action}'")

        post_by_org_by_repo_commit_body_files_item_type_1 = cls(
            path=path,
            action=action,
        )


        post_by_org_by_repo_commit_body_files_item_type_1.additional_properties = d
        return post_by_org_by_repo_commit_body_files_item_type_1

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Automatically generated by SST
# pylint: disable=all
# ruff: noqa
from typing import Any

class Resource:
    class App:
        name: str
        stage: str
    class AuthEnvelopePrivateKey:
        type: str
        value: str
    class AuthEnvelopePublicKey:
        type: str
        value: str
    class BetterAuthSecret:
        type: str
        value: str
    class Config:
        API_HOST: str
        API_PORT: float
        API_URL: str
        API_VCS_PORT: float
        APP_HOST: str
        APP_PORT: float
        APP_URL: str
        DEPOT_DEFAULT_REPO_SIZE_LIMIT: float
        DEPOT_MAX_REPO_NAME_LENGTH: float
        DEPOT_REPO_ROOT: str
        DEV: bool
        NORTHSTAR_HOST: str
        NORTHSTAR_PORT: float
        NORTHSTAR_URL: str
        STAGE: str
        SUPPORT_EMAIL: str
        UPSTREAM_KMS_KEY_ARN: str
        VCS_HOST: str
        VCS_PORT: float
        VCS_URL: str
        type: str
    class DepotDB:
        database: str
        host: str
        password: str
        port: float
        type: str
        username: str
    class DepotService:
        service: str
        type: str
        url: str
    class DepotVPC:
        bastion: str
        type: str
    class DepotWeb:
        type: str
        url: str
    class GoogleClientId:
        type: str
        value: str
    class GoogleClientSecret:
        type: str
        value: str
    class GrafanaKey:
        type: str
        value: str
    class NorthstarInternalToken:
        type: str
        value: str
    class SlackWaitlistWebhookUrl:
        type: str
        value: str
    class VcsInternalSecret:
        type: str
        value: str
    class VcsService:
        service: str
        type: str
        url: str
    class WWWSite:
        type: str
        url: str
    class Waitlist:
        name: str
        type: str




from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import cast

if TYPE_CHECKING:
  from ..models.get_by_org_by_repo_commit_response_200_commits_item import GetByOrgByRepoCommitResponse200CommitsItem





T = TypeVar("T", bound="GetByOrgByRepoCommitResponse200")



@_attrs_define
class GetByOrgByRepoCommitResponse200:
    """ 
        Attributes:
            next_cursor (None | str):
            has_more (bool):
            commits (list[GetByOrgByRepoCommitResponse200CommitsItem]):
     """

    next_cursor: None | str
    has_more: bool
    commits: list[GetByOrgByRepoCommitResponse200CommitsItem]
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        from ..models.get_by_org_by_repo_commit_response_200_commits_item import GetByOrgByRepoCommitResponse200CommitsItem
        next_cursor: None | str
        next_cursor = self.next_cursor

        has_more = self.has_more

        commits = []
        for commits_item_data in self.commits:
            commits_item = commits_item_data.to_dict()
            commits.append(commits_item)




        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "next_cursor": next_cursor,
            "has_more": has_more,
            "commits": commits,
        })

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.get_by_org_by_repo_commit_response_200_commits_item import GetByOrgByRepoCommitResponse200CommitsItem
        d = dict(src_dict)
        def _parse_next_cursor(data: object) -> None | str:
            if data is None:
                return data
            return cast(None | str, data)

        next_cursor = _parse_next_cursor(d.pop("next_cursor"))


        has_more = d.pop("has_more")

        commits = []
        _commits = d.pop("commits")
        for commits_item_data in (_commits):
            commits_item = GetByOrgByRepoCommitResponse200CommitsItem.from_dict(commits_item_data)



            commits.append(commits_item)


        get_by_org_by_repo_commit_response_200 = cls(
            next_cursor=next_cursor,
            has_more=has_more,
            commits=commits,
        )


        get_by_org_by_repo_commit_response_200.additional_properties = d
        return get_by_org_by_repo_commit_response_200

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, cast
from urllib.parse import quote

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response, UNSET
from ... import errors

from ...models.delete_by_org_api_key_by_id_response_200 import DeleteByOrgApiKeyByIdResponse200
from ...models.delete_by_org_api_key_by_id_response_400 import DeleteByOrgApiKeyByIdResponse400
from ...models.delete_by_org_api_key_by_id_response_401 import DeleteByOrgApiKeyByIdResponse401
from ...models.delete_by_org_api_key_by_id_response_403 import DeleteByOrgApiKeyByIdResponse403
from ...models.delete_by_org_api_key_by_id_response_404 import DeleteByOrgApiKeyByIdResponse404
from ...models.delete_by_org_api_key_by_id_response_406 import DeleteByOrgApiKeyByIdResponse406
from ...models.delete_by_org_api_key_by_id_response_409 import DeleteByOrgApiKeyByIdResponse409
from ...models.delete_by_org_api_key_by_id_response_500 import DeleteByOrgApiKeyByIdResponse500
from typing import cast



def _get_kwargs(
    org: str,
    id: str,

) -> dict[str, Any]:
    

    

    

    _kwargs: dict[str, Any] = {
        "method": "delete",
        "url": "/{org}/api-key/{id}".format(org=quote(str(org), safe=""),id=quote(str(id), safe=""),),
    }


    return _kwargs



def _parse_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> DeleteByOrgApiKeyByIdResponse200 | DeleteByOrgApiKeyByIdResponse400 | DeleteByOrgApiKeyByIdResponse401 | DeleteByOrgApiKeyByIdResponse403 | DeleteByOrgApiKeyByIdResponse404 | DeleteByOrgApiKeyByIdResponse406 | DeleteByOrgApiKeyByIdResponse409 | DeleteByOrgApiKeyByIdResponse500 | None:
    if response.status_code == 200:
        response_200 = DeleteByOrgApiKeyByIdResponse200.from_dict(response.json())



        return response_200

    if response.status_code == 400:
        response_400 = DeleteByOrgApiKeyByIdResponse400.from_dict(response.json())



        return response_400

    if response.status_code == 401:
        response_401 = DeleteByOrgApiKeyByIdResponse401.from_dict(response.json())



        return response_401

    if response.status_code == 403:
        response_403 = DeleteByOrgApiKeyByIdResponse403.from_dict(response.json())



        return response_403

    if response.status_code == 404:
        response_404 = DeleteByOrgApiKeyByIdResponse404.from_dict(response.json())



        return response_404

    if response.status_code == 406:
        response_406 = DeleteByOrgApiKeyByIdResponse406.from_dict(response.json())



        return response_406

    if response.status_code == 409:
        response_409 = DeleteByOrgApiKeyByIdResponse409.from_dict(response.json())



        return response_409

    if response.status_code == 500:
        response_500 = DeleteByOrgApiKeyByIdResponse500.from_dict(response.json())



        return response_500

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> Response[DeleteByOrgApiKeyByIdResponse200 | DeleteByOrgApiKeyByIdResponse400 | DeleteByOrgApiKeyByIdResponse401 | DeleteByOrgApiKeyByIdResponse403 | DeleteByOrgApiKeyByIdResponse404 | DeleteByOrgApiKeyByIdResponse406 | DeleteByOrgApiKeyByIdResponse409 | DeleteByOrgApiKeyByIdResponse500]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    org: str,
    id: str,
    *,
    client: AuthenticatedClient | Client,

) -> Response[DeleteByOrgApiKeyByIdResponse200 | DeleteByOrgApiKeyByIdResponse400 | DeleteByOrgApiKeyByIdResponse401 | DeleteByOrgApiKeyByIdResponse403 | DeleteByOrgApiKeyByIdResponse404 | DeleteByOrgApiKeyByIdResponse406 | DeleteByOrgApiKeyByIdResponse409 | DeleteByOrgApiKeyByIdResponse500]:
    """ Revoke API key

     Revoke an API key by its ID

    Args:
        org (str):
        id (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[DeleteByOrgApiKeyByIdResponse200 | DeleteByOrgApiKeyByIdResponse400 | DeleteByOrgApiKeyByIdResponse401 | DeleteByOrgApiKeyByIdResponse403 | DeleteByOrgApiKeyByIdResponse404 | DeleteByOrgApiKeyByIdResponse406 | DeleteByOrgApiKeyByIdResponse409 | DeleteByOrgApiKeyByIdResponse500]
     """


    kwargs = _get_kwargs(
        org=org,
id=id,

    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)

def sync(
    org: str,
    id: str,
    *,
    client: AuthenticatedClient | Client,

) -> DeleteByOrgApiKeyByIdResponse200 | DeleteByOrgApiKeyByIdResponse400 | DeleteByOrgApiKeyByIdResponse401 | DeleteByOrgApiKeyByIdResponse403 | DeleteByOrgApiKeyByIdResponse404 | DeleteByOrgApiKeyByIdResponse406 | DeleteByOrgApiKeyByIdResponse409 | DeleteByOrgApiKeyByIdResponse500 | None:
    """ Revoke API key

     Revoke an API key by its ID

    Args:
        org (str):
        id (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        DeleteByOrgApiKeyByIdResponse200 | DeleteByOrgApiKeyByIdResponse400 | DeleteByOrgApiKeyByIdResponse401 | DeleteByOrgApiKeyByIdResponse403 | DeleteByOrgApiKeyByIdResponse404 | DeleteByOrgApiKeyByIdResponse406 | DeleteByOrgApiKeyByIdResponse409 | DeleteByOrgApiKeyByIdResponse500
     """


    return sync_detailed(
        org=org,
id=id,
client=client,

    ).parsed

async def asyncio_detailed(
    org: str,
    id: str,
    *,
    client: AuthenticatedClient | Client,

) -> Response[DeleteByOrgApiKeyByIdResponse200 | DeleteByOrgApiKeyByIdResponse400 | DeleteByOrgApiKeyByIdResponse401 | DeleteByOrgApiKeyByIdResponse403 | DeleteByOrgApiKeyByIdResponse404 | DeleteByOrgApiKeyByIdResponse406 | DeleteByOrgApiKeyByIdResponse409 | DeleteByOrgApiKeyByIdResponse500]:
    """ Revoke API key

     Revoke an API key by its ID

    Args:
        org (str):
        id (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[DeleteByOrgApiKeyByIdResponse200 | DeleteByOrgApiKeyByIdResponse400 | DeleteByOrgApiKeyByIdResponse401 | DeleteByOrgApiKeyByIdResponse403 | DeleteByOrgApiKeyByIdResponse404 | DeleteByOrgApiKeyByIdResponse406 | DeleteByOrgApiKeyByIdResponse409 | DeleteByOrgApiKeyByIdResponse500]
     """


    kwargs = _get_kwargs(
        org=org,
id=id,

    )

    response = await client.get_async_httpx_client().request(
        **kwargs
    )

    return _build_response(client=client, response=response)

async def asyncio(
    org: str,
    id: str,
    *,
    client: AuthenticatedClient | Client,

) -> DeleteByOrgApiKeyByIdResponse200 | DeleteByOrgApiKeyByIdResponse400 | DeleteByOrgApiKeyByIdResponse401 | DeleteByOrgApiKeyByIdResponse403 | DeleteByOrgApiKeyByIdResponse404 | DeleteByOrgApiKeyByIdResponse406 | DeleteByOrgApiKeyByIdResponse409 | DeleteByOrgApiKeyByIdResponse500 | None:
    """ Revoke API key

     Revoke an API key by its ID

    Args:
        org (str):
        id (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        DeleteByOrgApiKeyByIdResponse200 | DeleteByOrgApiKeyByIdResponse400 | DeleteByOrgApiKeyByIdResponse401 | DeleteByOrgApiKeyByIdResponse403 | DeleteByOrgApiKeyByIdResponse404 | DeleteByOrgApiKeyByIdResponse406 | DeleteByOrgApiKeyByIdResponse409 | DeleteByOrgApiKeyByIdResponse500
     """


    return (await asyncio_detailed(
        org=org,
id=id,
client=client,

    )).parsed



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import cast






T = TypeVar("T", bound="PostByOrgApiKeyResponse201")



@_attrs_define
class PostByOrgApiKeyResponse201:
    """ 
        Attributes:
            id (str):
            key (str):
            name (None | str):
            scopes (list[str]):
            repo_ids (list[str] | None):
            created_at (str):
     """

    id: str
    key: str
    name: None | str
    scopes: list[str]
    repo_ids: list[str] | None
    created_at: str
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        id = self.id

        key = self.key

        name: None | str
        name = self.name

        scopes = self.scopes



        repo_ids: list[str] | None
        if isinstance(self.repo_ids, list):
            repo_ids = self.repo_ids


        else:
            repo_ids = self.repo_ids

        created_at = self.created_at


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "id": id,
            "key": key,
            "name": name,
            "scopes": scopes,
            "repo_ids": repo_ids,
            "created_at": created_at,
        })

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        id = d.pop("id")

        key = d.pop("key")

        def _parse_name(data: object) -> None | str:
            if data is None:
                return data
            return cast(None | str, data)

        name = _parse_name(d.pop("name"))


        scopes = cast(list[str], d.pop("scopes"))


        def _parse_repo_ids(data: object) -> list[str] | None:
            if data is None:
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                repo_ids_type_0 = cast(list[str], data)

                return repo_ids_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(list[str] | None, data)

        repo_ids = _parse_repo_ids(d.pop("repo_ids"))


        created_at = d.pop("created_at")

        post_by_org_api_key_response_201 = cls(
            id=id,
            key=key,
            name=name,
            scopes=scopes,
            repo_ids=repo_ids,
            created_at=created_at,
        )


        post_by_org_api_key_response_201.additional_properties = d
        return post_by_org_api_key_response_201

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import cast
from typing import Literal, cast

if TYPE_CHECKING:
  from ..models.get_by_org_by_repo_content_response_200_type_2_entries_item_type_0 import GetByOrgByRepoContentResponse200Type2EntriesItemType0
  from ..models.get_by_org_by_repo_content_response_200_type_2_entries_item_type_1 import GetByOrgByRepoContentResponse200Type2EntriesItemType1
  from ..models.get_by_org_by_repo_content_response_200_type_2_entries_item_type_2 import GetByOrgByRepoContentResponse200Type2EntriesItemType2





T = TypeVar("T", bound="GetByOrgByRepoContentResponse200Type2")



@_attrs_define
class GetByOrgByRepoContentResponse200Type2:
    """ 
        Attributes:
            type_ (Literal['dir']):
            name (str):
            path (str):
            sha (str):
            child_count (int):
            entries (list[GetByOrgByRepoContentResponse200Type2EntriesItemType0 |
                GetByOrgByRepoContentResponse200Type2EntriesItemType1 | GetByOrgByRepoContentResponse200Type2EntriesItemType2]):
            next_cursor (None | str):
            has_more (bool):
     """

    type_: Literal['dir']
    name: str
    path: str
    sha: str
    child_count: int
    entries: list[GetByOrgByRepoContentResponse200Type2EntriesItemType0 | GetByOrgByRepoContentResponse200Type2EntriesItemType1 | GetByOrgByRepoContentResponse200Type2EntriesItemType2]
    next_cursor: None | str
    has_more: bool
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        from ..models.get_by_org_by_repo_content_response_200_type_2_entries_item_type_1 import GetByOrgByRepoContentResponse200Type2EntriesItemType1
        from ..models.get_by_org_by_repo_content_response_200_type_2_entries_item_type_2 import GetByOrgByRepoContentResponse200Type2EntriesItemType2
        from ..models.get_by_org_by_repo_content_response_200_type_2_entries_item_type_0 import GetByOrgByRepoContentResponse200Type2EntriesItemType0
        type_ = self.type_

        name = self.name

        path = self.path

        sha = self.sha

        child_count = self.child_count

        entries = []
        for entries_item_data in self.entries:
            entries_item: dict[str, Any]
            if isinstance(entries_item_data, GetByOrgByRepoContentResponse200Type2EntriesItemType0):
                entries_item = entries_item_data.to_dict()
            elif isinstance(entries_item_data, GetByOrgByRepoContentResponse200Type2EntriesItemType1):
                entries_item = entries_item_data.to_dict()
            else:
                entries_item = entries_item_data.to_dict()

            entries.append(entries_item)



        next_cursor: None | str
        next_cursor = self.next_cursor

        has_more = self.has_more


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "type": type_,
            "name": name,
            "path": path,
            "sha": sha,
            "child_count": child_count,
            "entries": entries,
            "next_cursor": next_cursor,
            "has_more": has_more,
        })

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.get_by_org_by_repo_content_response_200_type_2_entries_item_type_0 import GetByOrgByRepoContentResponse200Type2EntriesItemType0
        from ..models.get_by_org_by_repo_content_response_200_type_2_entries_item_type_1 import GetByOrgByRepoContentResponse200Type2EntriesItemType1
        from ..models.get_by_org_by_repo_content_response_200_type_2_entries_item_type_2 import GetByOrgByRepoContentResponse200Type2EntriesItemType2
        d = dict(src_dict)
        type_ = cast(Literal['dir'] , d.pop("type"))
        if type_ != 'dir':
            raise ValueError(f"type must match const 'dir', got '{type_}'")

        name = d.pop("name")

        path = d.pop("path")

        sha = d.pop("sha")

        child_count = d.pop("child_count")

        entries = []
        _entries = d.pop("entries")
        for entries_item_data in (_entries):
            def _parse_entries_item(data: object) -> GetByOrgByRepoContentResponse200Type2EntriesItemType0 | GetByOrgByRepoContentResponse200Type2EntriesItemType1 | GetByOrgByRepoContentResponse200Type2EntriesItemType2:
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    entries_item_type_0 = GetByOrgByRepoContentResponse200Type2EntriesItemType0.from_dict(data)



                    return entries_item_type_0
                except (TypeError, ValueError, AttributeError, KeyError):
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    entries_item_type_1 = GetByOrgByRepoContentResponse200Type2EntriesItemType1.from_dict(data)



                    return entries_item_type_1
                except (TypeError, ValueError, AttributeError, KeyError):
                    pass
                if not isinstance(data, dict):
                    raise TypeError()
                entries_item_type_2 = GetByOrgByRepoContentResponse200Type2EntriesItemType2.from_dict(data)



                return entries_item_type_2

            entries_item = _parse_entries_item(entries_item_data)

            entries.append(entries_item)


        def _parse_next_cursor(data: object) -> None | str:
            if data is None:
                return data
            return cast(None | str, data)

        next_cursor = _parse_next_cursor(d.pop("next_cursor"))


        has_more = d.pop("has_more")

        get_by_org_by_repo_content_response_200_type_2 = cls(
            type_=type_,
            name=name,
            path=path,
            sha=sha,
            child_count=child_count,
            entries=entries,
            next_cursor=next_cursor,
            has_more=has_more,
        )


        get_by_org_by_repo_content_response_200_type_2.additional_properties = d
        return get_by_org_by_repo_content_response_200_type_2

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.post_by_org_by_repo_webhook_body_events_item import PostByOrgByRepoWebhookBodyEventsItem
from ..types import UNSET, Unset
from typing import cast






T = TypeVar("T", bound="PostByOrgByRepoWebhookBody")



@_attrs_define
class PostByOrgByRepoWebhookBody:
    """ 
        Attributes:
            url (str):
            events (list[PostByOrgByRepoWebhookBodyEventsItem] | Unset):
            branches (list[str] | Unset):
            globs (list[str] | Unset):
            secret (str | Unset):
     """

    url: str
    events: list[PostByOrgByRepoWebhookBodyEventsItem] | Unset = UNSET
    branches: list[str] | Unset = UNSET
    globs: list[str] | Unset = UNSET
    secret: str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        url = self.url

        events: list[str] | Unset = UNSET
        if not isinstance(self.events, Unset):
            events = []
            for events_item_data in self.events:
                events_item = events_item_data.value
                events.append(events_item)



        branches: list[str] | Unset = UNSET
        if not isinstance(self.branches, Unset):
            branches = self.branches



        globs: list[str] | Unset = UNSET
        if not isinstance(self.globs, Unset):
            globs = self.globs



        secret = self.secret


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "url": url,
        })
        if events is not UNSET:
            field_dict["events"] = events
        if branches is not UNSET:
            field_dict["branches"] = branches
        if globs is not UNSET:
            field_dict["globs"] = globs
        if secret is not UNSET:
            field_dict["secret"] = secret

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        url = d.pop("url")

        _events = d.pop("events", UNSET)
        events: list[PostByOrgByRepoWebhookBodyEventsItem] | Unset = UNSET
        if _events is not UNSET:
            events = []
            for events_item_data in _events:
                events_item = PostByOrgByRepoWebhookBodyEventsItem(events_item_data)



                events.append(events_item)


        branches = cast(list[str], d.pop("branches", UNSET))


        globs = cast(list[str], d.pop("globs", UNSET))


        secret = d.pop("secret", UNSET)

        post_by_org_by_repo_webhook_body = cls(
            url=url,
            events=events,
            branches=branches,
            globs=globs,
            secret=secret,
        )


        post_by_org_by_repo_webhook_body.additional_properties = d
        return post_by_org_by_repo_webhook_body

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import cast

if TYPE_CHECKING:
  from ..models.post_by_org_by_repo_commit_body_author import PostByOrgByRepoCommitBodyAuthor
  from ..models.post_by_org_by_repo_commit_body_committer import PostByOrgByRepoCommitBodyCommitter
  from ..models.post_by_org_by_repo_commit_body_files_item_type_0 import PostByOrgByRepoCommitBodyFilesItemType0
  from ..models.post_by_org_by_repo_commit_body_files_item_type_1 import PostByOrgByRepoCommitBodyFilesItemType1





T = TypeVar("T", bound="PostByOrgByRepoCommitBody")



@_attrs_define
class PostByOrgByRepoCommitBody:
    """ 
        Attributes:
            branch (str):
            message (str):
            author (PostByOrgByRepoCommitBodyAuthor):
            files (list[PostByOrgByRepoCommitBodyFilesItemType0 | PostByOrgByRepoCommitBodyFilesItemType1]):
            committer (PostByOrgByRepoCommitBodyCommitter | Unset):
            base_sha (str | Unset):
     """

    branch: str
    message: str
    author: PostByOrgByRepoCommitBodyAuthor
    files: list[PostByOrgByRepoCommitBodyFilesItemType0 | PostByOrgByRepoCommitBodyFilesItemType1]
    committer: PostByOrgByRepoCommitBodyCommitter | Unset = UNSET
    base_sha: str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        from ..models.post_by_org_by_repo_commit_body_author import PostByOrgByRepoCommitBodyAuthor
        from ..models.post_by_org_by_repo_commit_body_committer import PostByOrgByRepoCommitBodyCommitter
        from ..models.post_by_org_by_repo_commit_body_files_item_type_1 import PostByOrgByRepoCommitBodyFilesItemType1
        from ..models.post_by_org_by_repo_commit_body_files_item_type_0 import PostByOrgByRepoCommitBodyFilesItemType0
        branch = self.branch

        message = self.message

        author = self.author.to_dict()

        files = []
        for files_item_data in self.files:
            files_item: dict[str, Any]
            if isinstance(files_item_data, PostByOrgByRepoCommitBodyFilesItemType0):
                files_item = files_item_data.to_dict()
            else:
                files_item = files_item_data.to_dict()

            files.append(files_item)



        committer: dict[str, Any] | Unset = UNSET
        if not isinstance(self.committer, Unset):
            committer = self.committer.to_dict()

        base_sha = self.base_sha


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "branch": branch,
            "message": message,
            "author": author,
            "files": files,
        })
        if committer is not UNSET:
            field_dict["committer"] = committer
        if base_sha is not UNSET:
            field_dict["base_sha"] = base_sha

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.post_by_org_by_repo_commit_body_author import PostByOrgByRepoCommitBodyAuthor
        from ..models.post_by_org_by_repo_commit_body_committer import PostByOrgByRepoCommitBodyCommitter
        from ..models.post_by_org_by_repo_commit_body_files_item_type_0 import PostByOrgByRepoCommitBodyFilesItemType0
        from ..models.post_by_org_by_repo_commit_body_files_item_type_1 import PostByOrgByRepoCommitBodyFilesItemType1
        d = dict(src_dict)
        branch = d.pop("branch")

        message = d.pop("message")

        author = PostByOrgByRepoCommitBodyAuthor.from_dict(d.pop("author"))




        files = []
        _files = d.pop("files")
        for files_item_data in (_files):
            def _parse_files_item(data: object) -> PostByOrgByRepoCommitBodyFilesItemType0 | PostByOrgByRepoCommitBodyFilesItemType1:
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    files_item_type_0 = PostByOrgByRepoCommitBodyFilesItemType0.from_dict(data)



                    return files_item_type_0
                except (TypeError, ValueError, AttributeError, KeyError):
                    pass
                if not isinstance(data, dict):
                    raise TypeError()
                files_item_type_1 = PostByOrgByRepoCommitBodyFilesItemType1.from_dict(data)



                return files_item_type_1

            files_item = _parse_files_item(files_item_data)

            files.append(files_item)


        _committer = d.pop("committer", UNSET)
        committer: PostByOrgByRepoCommitBodyCommitter | Unset
        if isinstance(_committer,  Unset):
            committer = UNSET
        else:
            committer = PostByOrgByRepoCommitBodyCommitter.from_dict(_committer)




        base_sha = d.pop("base_sha", UNSET)

        post_by_org_by_repo_commit_body = cls(
            branch=branch,
            message=message,
            author=author,
            files=files,
            committer=committer,
            base_sha=base_sha,
        )


        post_by_org_by_repo_commit_body.additional_properties = d
        return post_by_org_by_repo_commit_body

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.post_by_org_repo_body_upstream_autosync_type_0_resolution_strategy import PostByOrgRepoBodyUpstreamAutosyncType0ResolutionStrategy
from ..types import UNSET, Unset
from typing import Literal, cast






T = TypeVar("T", bound="PostByOrgRepoBodyUpstreamAutosyncType0")



@_attrs_define
class PostByOrgRepoBodyUpstreamAutosyncType0:
    """ 
        Attributes:
            type_ (Literal['poll']):
            period (int): Polling period in seconds (60s to 24.8d)
            resolution_strategy (PostByOrgRepoBodyUpstreamAutosyncType0ResolutionStrategy | Unset): Conflict resolution
                strategy. "none" means sync will fail on conflicts. Default:
                PostByOrgRepoBodyUpstreamAutosyncType0ResolutionStrategy.NONE.
     """

    type_: Literal['poll']
    period: int
    resolution_strategy: PostByOrgRepoBodyUpstreamAutosyncType0ResolutionStrategy | Unset = PostByOrgRepoBodyUpstreamAutosyncType0ResolutionStrategy.NONE
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        type_ = self.type_

        period = self.period

        resolution_strategy: str | Unset = UNSET
        if not isinstance(self.resolution_strategy, Unset):
            resolution_strategy = self.resolution_strategy.value



        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "type": type_,
            "period": period,
        })
        if resolution_strategy is not UNSET:
            field_dict["resolution_strategy"] = resolution_strategy

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        type_ = cast(Literal['poll'] , d.pop("type"))
        if type_ != 'poll':
            raise ValueError(f"type must match const 'poll', got '{type_}'")

        period = d.pop("period")

        _resolution_strategy = d.pop("resolution_strategy", UNSET)
        resolution_strategy: PostByOrgRepoBodyUpstreamAutosyncType0ResolutionStrategy | Unset
        if isinstance(_resolution_strategy,  Unset):
            resolution_strategy = UNSET
        else:
            resolution_strategy = PostByOrgRepoBodyUpstreamAutosyncType0ResolutionStrategy(_resolution_strategy)




        post_by_org_repo_body_upstream_autosync_type_0 = cls(
            type_=type_,
            period=period,
            resolution_strategy=resolution_strategy,
        )


        post_by_org_repo_body_upstream_autosync_type_0.additional_properties = d
        return post_by_org_repo_body_upstream_autosync_type_0

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, cast
from urllib.parse import quote

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response, UNSET
from ... import errors

from ...models.post_by_org_by_repo_webhook_body import PostByOrgByRepoWebhookBody
from ...models.post_by_org_by_repo_webhook_response_201 import PostByOrgByRepoWebhookResponse201
from ...models.post_by_org_by_repo_webhook_response_400 import PostByOrgByRepoWebhookResponse400
from ...models.post_by_org_by_repo_webhook_response_401 import PostByOrgByRepoWebhookResponse401
from ...models.post_by_org_by_repo_webhook_response_403 import PostByOrgByRepoWebhookResponse403
from ...models.post_by_org_by_repo_webhook_response_404 import PostByOrgByRepoWebhookResponse404
from ...models.post_by_org_by_repo_webhook_response_406 import PostByOrgByRepoWebhookResponse406
from ...models.post_by_org_by_repo_webhook_response_409 import PostByOrgByRepoWebhookResponse409
from ...models.post_by_org_by_repo_webhook_response_500 import PostByOrgByRepoWebhookResponse500
from typing import cast



def _get_kwargs(
    org: str,
    repo: str,
    *,
    body: PostByOrgByRepoWebhookBody,

) -> dict[str, Any]:
    headers: dict[str, Any] = {}


    

    

    _kwargs: dict[str, Any] = {
        "method": "post",
        "url": "/{org}/{repo}/webhook".format(org=quote(str(org), safe=""),repo=quote(str(repo), safe=""),),
    }

    _kwargs["json"] = body.to_dict()


    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs



def _parse_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> PostByOrgByRepoWebhookResponse201 | PostByOrgByRepoWebhookResponse400 | PostByOrgByRepoWebhookResponse401 | PostByOrgByRepoWebhookResponse403 | PostByOrgByRepoWebhookResponse404 | PostByOrgByRepoWebhookResponse406 | PostByOrgByRepoWebhookResponse409 | PostByOrgByRepoWebhookResponse500 | None:
    if response.status_code == 201:
        response_201 = PostByOrgByRepoWebhookResponse201.from_dict(response.json())



        return response_201

    if response.status_code == 400:
        response_400 = PostByOrgByRepoWebhookResponse400.from_dict(response.json())



        return response_400

    if response.status_code == 401:
        response_401 = PostByOrgByRepoWebhookResponse401.from_dict(response.json())



        return response_401

    if response.status_code == 403:
        response_403 = PostByOrgByRepoWebhookResponse403.from_dict(response.json())



        return response_403

    if response.status_code == 404:
        response_404 = PostByOrgByRepoWebhookResponse404.from_dict(response.json())



        return response_404

    if response.status_code == 406:
        response_406 = PostByOrgByRepoWebhookResponse406.from_dict(response.json())



        return response_406

    if response.status_code == 409:
        response_409 = PostByOrgByRepoWebhookResponse409.from_dict(response.json())



        return response_409

    if response.status_code == 500:
        response_500 = PostByOrgByRepoWebhookResponse500.from_dict(response.json())



        return response_500

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> Response[PostByOrgByRepoWebhookResponse201 | PostByOrgByRepoWebhookResponse400 | PostByOrgByRepoWebhookResponse401 | PostByOrgByRepoWebhookResponse403 | PostByOrgByRepoWebhookResponse404 | PostByOrgByRepoWebhookResponse406 | PostByOrgByRepoWebhookResponse409 | PostByOrgByRepoWebhookResponse500]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    org: str,
    repo: str,
    *,
    client: AuthenticatedClient | Client,
    body: PostByOrgByRepoWebhookBody,

) -> Response[PostByOrgByRepoWebhookResponse201 | PostByOrgByRepoWebhookResponse400 | PostByOrgByRepoWebhookResponse401 | PostByOrgByRepoWebhookResponse403 | PostByOrgByRepoWebhookResponse404 | PostByOrgByRepoWebhookResponse406 | PostByOrgByRepoWebhookResponse409 | PostByOrgByRepoWebhookResponse500]:
    """ Create webhook

     Create a webhook for a repository

    Args:
        org (str):
        repo (str):
        body (PostByOrgByRepoWebhookBody):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[PostByOrgByRepoWebhookResponse201 | PostByOrgByRepoWebhookResponse400 | PostByOrgByRepoWebhookResponse401 | PostByOrgByRepoWebhookResponse403 | PostByOrgByRepoWebhookResponse404 | PostByOrgByRepoWebhookResponse406 | PostByOrgByRepoWebhookResponse409 | PostByOrgByRepoWebhookResponse500]
     """


    kwargs = _get_kwargs(
        org=org,
repo=repo,
body=body,

    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)

def sync(
    org: str,
    repo: str,
    *,
    client: AuthenticatedClient | Client,
    body: PostByOrgByRepoWebhookBody,

) -> PostByOrgByRepoWebhookResponse201 | PostByOrgByRepoWebhookResponse400 | PostByOrgByRepoWebhookResponse401 | PostByOrgByRepoWebhookResponse403 | PostByOrgByRepoWebhookResponse404 | PostByOrgByRepoWebhookResponse406 | PostByOrgByRepoWebhookResponse409 | PostByOrgByRepoWebhookResponse500 | None:
    """ Create webhook

     Create a webhook for a repository

    Args:
        org (str):
        repo (str):
        body (PostByOrgByRepoWebhookBody):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        PostByOrgByRepoWebhookResponse201 | PostByOrgByRepoWebhookResponse400 | PostByOrgByRepoWebhookResponse401 | PostByOrgByRepoWebhookResponse403 | PostByOrgByRepoWebhookResponse404 | PostByOrgByRepoWebhookResponse406 | PostByOrgByRepoWebhookResponse409 | PostByOrgByRepoWebhookResponse500
     """


    return sync_detailed(
        org=org,
repo=repo,
client=client,
body=body,

    ).parsed

async def asyncio_detailed(
    org: str,
    repo: str,
    *,
    client: AuthenticatedClient | Client,
    body: PostByOrgByRepoWebhookBody,

) -> Response[PostByOrgByRepoWebhookResponse201 | PostByOrgByRepoWebhookResponse400 | PostByOrgByRepoWebhookResponse401 | PostByOrgByRepoWebhookResponse403 | PostByOrgByRepoWebhookResponse404 | PostByOrgByRepoWebhookResponse406 | PostByOrgByRepoWebhookResponse409 | PostByOrgByRepoWebhookResponse500]:
    """ Create webhook

     Create a webhook for a repository

    Args:
        org (str):
        repo (str):
        body (PostByOrgByRepoWebhookBody):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[PostByOrgByRepoWebhookResponse201 | PostByOrgByRepoWebhookResponse400 | PostByOrgByRepoWebhookResponse401 | PostByOrgByRepoWebhookResponse403 | PostByOrgByRepoWebhookResponse404 | PostByOrgByRepoWebhookResponse406 | PostByOrgByRepoWebhookResponse409 | PostByOrgByRepoWebhookResponse500]
     """


    kwargs = _get_kwargs(
        org=org,
repo=repo,
body=body,

    )

    response = await client.get_async_httpx_client().request(
        **kwargs
    )

    return _build_response(client=client, response=response)

async def asyncio(
    org: str,
    repo: str,
    *,
    client: AuthenticatedClient | Client,
    body: PostByOrgByRepoWebhookBody,

) -> PostByOrgByRepoWebhookResponse201 | PostByOrgByRepoWebhookResponse400 | PostByOrgByRepoWebhookResponse401 | PostByOrgByRepoWebhookResponse403 | PostByOrgByRepoWebhookResponse404 | PostByOrgByRepoWebhookResponse406 | PostByOrgByRepoWebhookResponse409 | PostByOrgByRepoWebhookResponse500 | None:
    """ Create webhook

     Create a webhook for a repository

    Args:
        org (str):
        repo (str):
        body (PostByOrgByRepoWebhookBody):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        PostByOrgByRepoWebhookResponse201 | PostByOrgByRepoWebhookResponse400 | PostByOrgByRepoWebhookResponse401 | PostByOrgByRepoWebhookResponse403 | PostByOrgByRepoWebhookResponse404 | PostByOrgByRepoWebhookResponse406 | PostByOrgByRepoWebhookResponse409 | PostByOrgByRepoWebhookResponse500
     """


    return (await asyncio_detailed(
        org=org,
repo=repo,
client=client,
body=body,

    )).parsed



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.get_by_org_by_repo_webhook_response_200_webhooks_item_events_item import GetByOrgByRepoWebhookResponse200WebhooksItemEventsItem
from typing import cast






T = TypeVar("T", bound="GetByOrgByRepoWebhookResponse200WebhooksItem")



@_attrs_define
class GetByOrgByRepoWebhookResponse200WebhooksItem:
    """ 
        Attributes:
            id (str):
            url (str):
            events (list[GetByOrgByRepoWebhookResponse200WebhooksItemEventsItem]):
            branches (list[str] | None):
            globs (list[str] | None):
            created_at (str):
            updated_at (str):
     """

    id: str
    url: str
    events: list[GetByOrgByRepoWebhookResponse200WebhooksItemEventsItem]
    branches: list[str] | None
    globs: list[str] | None
    created_at: str
    updated_at: str
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        id = self.id

        url = self.url

        events = []
        for events_item_data in self.events:
            events_item = events_item_data.value
            events.append(events_item)



        branches: list[str] | None
        if isinstance(self.branches, list):
            branches = self.branches


        else:
            branches = self.branches

        globs: list[str] | None
        if isinstance(self.globs, list):
            globs = self.globs


        else:
            globs = self.globs

        created_at = self.created_at

        updated_at = self.updated_at


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "id": id,
            "url": url,
            "events": events,
            "branches": branches,
            "globs": globs,
            "created_at": created_at,
            "updated_at": updated_at,
        })

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        id = d.pop("id")

        url = d.pop("url")

        events = []
        _events = d.pop("events")
        for events_item_data in (_events):
            events_item = GetByOrgByRepoWebhookResponse200WebhooksItemEventsItem(events_item_data)



            events.append(events_item)


        def _parse_branches(data: object) -> list[str] | None:
            if data is None:
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                branches_type_0 = cast(list[str], data)

                return branches_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(list[str] | None, data)

        branches = _parse_branches(d.pop("branches"))


        def _parse_globs(data: object) -> list[str] | None:
            if data is None:
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                globs_type_0 = cast(list[str], data)

                return globs_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(list[str] | None, data)

        globs = _parse_globs(d.pop("globs"))


        created_at = d.pop("created_at")

        updated_at = d.pop("updated_at")

        get_by_org_by_repo_webhook_response_200_webhooks_item = cls(
            id=id,
            url=url,
            events=events,
            branches=branches,
            globs=globs,
            created_at=created_at,
            updated_at=updated_at,
        )


        get_by_org_by_repo_webhook_response_200_webhooks_item.additional_properties = d
        return get_by_org_by_repo_webhook_response_200_webhooks_item

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.post_by_org_repo_bulk_tags_body_action import PostByOrgRepoBulkTagsBodyAction
from ..types import UNSET, Unset
from typing import cast






T = TypeVar("T", bound="PostByOrgRepoBulkTagsBody")



@_attrs_define
class PostByOrgRepoBulkTagsBody:
    """ 
        Attributes:
            repo_ids (list[str]):
            action (PostByOrgRepoBulkTagsBodyAction):
            key (str):
            value (str | Unset):
     """

    repo_ids: list[str]
    action: PostByOrgRepoBulkTagsBodyAction
    key: str
    value: str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        repo_ids = self.repo_ids



        action = self.action.value

        key = self.key

        value = self.value


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "repo_ids": repo_ids,
            "action": action,
            "key": key,
        })
        if value is not UNSET:
            field_dict["value"] = value

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        repo_ids = cast(list[str], d.pop("repo_ids"))


        action = PostByOrgRepoBulkTagsBodyAction(d.pop("action"))




        key = d.pop("key")

        value = d.pop("value", UNSET)

        post_by_org_repo_bulk_tags_body = cls(
            repo_ids=repo_ids,
            action=action,
            key=key,
            value=value,
        )


        post_by_org_repo_bulk_tags_body.additional_properties = d
        return post_by_org_repo_bulk_tags_body

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
